@@ -137,7 +137,10 @@ srl_build_decoder_struct(pTHX_ HV *opt)
     /* load options */
     if (opt != NULL) {
         if ( (svp = hv_fetchs(opt, "refuse_snappy", 0)) && SvTRUE(*svp))
-          dec->flags |= SRL_F_DECODER_REFUSE_SNAPPY;
+          SRL_DEC_SET_OPTION(dec, SRL_F_DECODER_REFUSE_SNAPPY);
+
+        if ( (svp = hv_fetchs(opt, "refuse_objects", 0)) && SvTRUE(*svp))
+          SRL_DEC_SET_OPTION(dec, SRL_F_DECODER_REFUSE_OBJECTS);
     }
 
     return dec;
@@ -462,7 +465,7 @@ srl_read_varint_uv_count(pTHX_ srl_decoder_t *dec, const char * const errstr)
 {
     UV len= srl_read_varint_uv(aTHX_ dec);
     if (len > I32_MAX) {
-        ERRORf3("Corrupterd packet%s. Count %lu exceeds I32_MAX (%li), which is impossible.",
+        ERRORf3("Corrupted packet%s. Count %lu exceeds I32_MAX (%i), which is impossible.",
                 errstr, len, I32_MAX);
     }
     return len;
@@ -724,7 +727,7 @@ srl_read_refp(pTHX_ srl_decoder_t *dec, SV* into)
     /* something we did before */
     UV item= srl_read_varint_uv_offset(aTHX_ dec, " while reading REFP tag");
     SV *referent= srl_fetch_item(aTHX_ dec, item, "REFP");
-    SvREFCNT_inc(referent);
+    (void)SvREFCNT_inc(referent);
 
     SRL_ASSERT_TYPE_FOR_RV(into);
     SvTEMP_off(referent);
@@ -753,8 +756,7 @@ srl_read_weaken(pTHX_ srl_decoder_t *dec, SV* into)
     if (expect_true( SvREFCNT(referent)==1 )) {
         if (expect_false( !dec->weakref_av ))
             dec->weakref_av= newAV();
-        SvREFCNT_inc(referent);
-        av_push(dec->weakref_av, referent);
+        av_push(dec->weakref_av, SvREFCNT_inc(referent));
         SRL_DEC_SET_OPTION(dec, SRL_F_DECODER_NEEDS_FINALIZE);
     }
     sv_rvweaken(into);
@@ -764,20 +766,24 @@ static SRL_INLINE void
 srl_read_objectv(pTHX_ srl_decoder_t *dec, SV* into)
 {
     AV *av= NULL;
-    STRLEN ofs= srl_read_varint_uv_offset(aTHX_ dec," while reading OBJECTV classname");
+    STRLEN ofs;
+
+    if (SRL_DEC_HAVE_OPTION(dec, SRL_F_DECODER_REFUSE_OBJECTS))
+        ERROR_REFUSE_OBJECT();
+
+    ofs= srl_read_varint_uv_offset(aTHX_ dec," while reading OBJECTV classname");
 
     if ( !dec->ref_bless_av)
-        ERROR("Corrupt packet. OBJECTV used without preceding OBJECTV of any kind");
+        ERROR("Corrupted packet. OBJECTV used without preceding OBJECT to define classname");
     av= (AV *)PTABLE_fetch(dec->ref_bless_av, (void *)ofs);
     if (NULL == av) {
-        ERRORf1("Corrupt packet. OBJECTV references unknown classname offset: %lu", ofs);
+        ERRORf1("Corrupted packet. OBJECTV references unknown classname offset: %lu", ofs);
     }
     /* now deparse the thing we are going to bless */
     srl_read_single_value(aTHX_ dec, into);
 
     /* and also stuff it into the av - we dont have to do any more book-keeping */
-    SvREFCNT_inc(into);
-    av_push(av, into);
+    av_push(av, SvREFCNT_inc(into));
 }
 
 static SRL_INLINE void
@@ -791,6 +797,10 @@ srl_read_object(pTHX_ srl_decoder_t *dec, SV* into)
     I32 flags= GV_ADD;
     U8 tag;
     U8 *from;
+
+    if (SRL_DEC_HAVE_OPTION(dec, SRL_F_DECODER_REFUSE_OBJECTS))
+        ERROR_REFUSE_OBJECT();
+
     /* now find the class name - first check if this is a copy op
      * this is bit tricky, as we could have a copy of a raw string
      * we could also have a copy of a previously mentioned class
@@ -870,8 +880,7 @@ srl_read_object(pTHX_ srl_decoder_t *dec, SV* into)
      * we really dont want to trigger DESTROY methods from a partial
      * deparse. So we insert the item into an array to be blessed later */
     SRL_DEC_SET_OPTION(dec, SRL_F_DECODER_NEEDS_FINALIZE);
-    SvREFCNT_inc(into);
-    av_push(av, into);
+    av_push(av, SvREFCNT_inc(into));
 
     /* now deparse the thing we are going to bless */
     srl_read_single_value(aTHX_ dec, into);
@@ -1004,8 +1013,7 @@ srl_read_alias(pTHX_ srl_decoder_t *dec)
 {
     UV item= srl_read_varint_uv_offset(aTHX_ dec," while reading ALIAS tag");
     SV *referent= srl_fetch_item(aTHX_ dec, item, "ALIAS");
-    SvREFCNT_inc(referent);
-    return referent;
+    return SvREFCNT_inc(referent);
 }
 
 static SRL_INLINE void
